@@ -21,14 +21,16 @@ limitations under the License.
 
 """
 
-from livestreamer.compat import str, bytes, urlparse, urljoin, unquote
+from livestreamer.compat import str, bytes, urlparse, urljoin, unquote, parse_qs
 from livestreamer.plugins import Plugin, PluginError, NoStreamsError
 from livestreamer.stream import HTTPStream
 from livestreamer.utils import urlget, urlopen
 from livestreamer.options import Options
 
+import socket
 import re
 import requests
+import time
 import xml.dom.minidom
 
 class GomTV(Plugin):
@@ -36,6 +38,8 @@ class GomTV(Plugin):
     LiveURL = BaseURL + "/main/goLive.gom"
     LoginURL = "https://ssl.gomtv.net/userinfo/loginProcess.gom"
     LoginCheckURL = BaseURL + "/forum/list.gom?m=my"
+    GOXVODURL = BaseURL + "/gox/ggox.gom"
+    KeyCheckPort = 63800
 
     LoginHeaders = {
         "Referer": BaseURL
@@ -45,7 +49,6 @@ class GomTV(Plugin):
         "User-Agent": "KPeerClient"
     }
 
-
     options = Options({
         "cookie": None,
         "username": None,
@@ -74,23 +77,77 @@ class GomTV(Plugin):
         else:
             self._authenticate(options.get("username"), options.get("password"))
 
+        if "/vod/" in self.url:
+            return self._get_vod_streams()
+        else:
+            return self._get_live_streams()
+
+    def _get_vod_streams(self):
+        res = urlget(self.url)
+        flashvars = re.search("FlashVars=\"(.+?)\"", res.text)
+
+        if not flashvars:
+            raise PluginError("Unable to find flashvars on page")
+
+        flashvars = parse_qs(flashvars.group(1))
+        for var in ("vjoinid", "conid", "leagueid"):
+            if not var in flashvars:
+                raise PluginError(("Missing key '{0}' in flashvars").format(var))
+
+        streams = {}
+        qualities = ["SQ", "HQ"]
+
+        for quality in qualities:
+            params = dict(leagueid=flashvars["leagueid"][0], vjoinid=flashvars["vjoinid"][0],
+                          conid=flashvars["conid"][0], title="", ref="",
+                          tmpstamp=int(time.time()), strLevel=quality)
+            res = urlget(self.GOXVODURL, params=params, session=self.rsession)
+
+            if res.text != "1002" and len(res.text) > 0:
+                gox = self._parse_gox_file(res.text)
+                entry = gox[0]
+
+                nokey = False
+                for var in ("NODEIP", "NODEID", "UNO", "USERIP"):
+                    if not var in entry:
+                        nokey = True
+
+                if nokey:
+                    self.logger.warning("Unable to fetch key, make sure that you have access to this VOD")
+                    continue
+
+                key = self._check_vod_key(entry["NODEIP"], entry["NODEID"], entry["UNO"],
+                                          entry["USERIP"])
+
+                streams[quality.lower()] = HTTPStream(self.session, gox[0]["REF"],
+                                                      params=dict(key=key),
+                                                      headers=self.StreamHeaders)
+
+
+        return streams
+
+
+    def _get_live_streams(self):
         streams = {}
         qualities = ["HQ", "SQ", "HQTest", "SQTest"]
 
         res = self._get_live_page(self.url)
-        urls = self._find_stream_urls(res.text)
+        goxurl = self._find_gox_url(res.text)
+
+        if not goxurl:
+            raise PluginError("Unable to find GOX URL")
 
         for quality in qualities:
-            for url in urls:
-                # Grab the response of the URL listed on the Live page for a stream
-                url = url.format(quality=quality)
-                res = urlget(url, session=self.rsession)
+            # Grab the response of the URL listed on the Live page for a stream
+            url = goxurl.format(quality=quality)
+            res = urlget(url, session=self.rsession)
+
+            # The response for the GOX XML if an incorrect stream quality is chosen is 1002.
+            if res.text != "1002" and len(res.text) > 0:
+                gox = self._parse_gox_file(res.text)
+                streams[quality.lower()] = HTTPStream(self.session, gox[0]["REF"],
+                                                      headers=self.StreamHeaders)
 
-                # The response for the GOX XML if an incorrect stream quality is chosen is 1002.
-                if res.text != "1002" and len(res.text) > 0:
-                    streamurl = self._parse_gox_file(res.text)
-                    streams[quality.lower()] = HTTPStream(self.session, streamurl,
-                                                          headers=self.StreamHeaders)
 
         return streams
 
@@ -127,6 +184,37 @@ class GomTV(Plugin):
             username = self.rsession.cookies["SES_USERNICK"]
             self.logger.info(("Successfully logged in as {0}").format(username))
 
+        if username and password:
+            cookie = ""
+            for v in ("SES_USERNO", "SES_STATE", "SES_MEMBERNICK", "SES_USERNICK"):
+                if v in self.rsession.cookies:
+                    cookie += "{0}={1}; ".format(v, self.rsession.cookies[v])
+            self.logger.info("Cookie for reusing this session: {0}", cookie)
+
+    def _check_vod_key(self, nodeip, nodeid, userno, userip):
+        try:
+            conn = socket.create_connection((nodeip, self.KeyCheckPort), timeout=15)
+        except socket.error as err:
+            raise PluginError(("Failed to connect to key check server: {0}").format(str(err)))
+
+        msg = "Login,0,{userno},{nodeid},{userip}\n".format(nodeid=nodeid, userno=userno,
+                                                          userip=userip)
+
+        try:
+            conn.sendall(bytes(msg, "ascii"))
+            res = conn.recv(4096)
+        except IOError as err:
+            raise PluginError(("Failed to communicate with key check server: {0}").format(str(err)))
+
+        if len(res) == 0:
+            raise PluginError("Empty response from key check server")
+
+        conn.close()
+
+        res = str(res, "ascii").strip().split(",")
+
+        return res[-1]
+
     def _get_event_url(self, prefix, data):
         match = re.search(' \"(.*)\";', data)
 
@@ -151,7 +239,7 @@ class GomTV(Plugin):
 
         return res
 
-    def _find_stream_urls(self, data):
+    def _find_gox_url(self, data):
         url = None
 
         # Parsing through the live page for a link to the gox XML file.
@@ -174,46 +262,45 @@ class GomTV(Plugin):
         except AttributeError:
             raise PluginError("Unable to find the stream title on the Live page")
 
-        # Check for multiple streams going at the same time, and extract the conid and the title
-        # Those streams have the class "live_now"
-        patternlive = '<a\shref=\"/live/index.gom\?conid=(?P<conid>\d+)\"\sclass=\"live_now\"\stitle=\"(?P<title>[^\"]+)'
-        streams = re.findall(patternlive, data)
+        return url
 
-        if len(streams) > 1:
-            urls = []
-            for stream in streams:
-                # Modify the urlFromHTML according to the user
-                singleurl = re.sub("conid=\d+", "conid=" + stream[0], url)
-                singletitlehtml = "+".join(stream[0].split(" "))
-                singleurl = re.sub("title=[\w|.|+]*", "title=" + singletitlehtml, singleurl)
-                urls.append(singleurl)
+    def _get_node_text(self, element):
+        res = []
+        for node in element.childNodes:
+            if node.nodeType == node.TEXT_NODE:
+                res.append(node.data)
 
-            return urls
+        if len(res) == 0:
+            return None
         else:
-            if url is None:
-                return []
-            else:
-                return [url]
+            return "".join(res)
 
     def _parse_gox_file(self, data):
-        # Grabbing the gomcmd URL
         try:
-            patternstream = '<REF href="([^"]*)"\s*/>'
-            match = re.search(patternstream, data).group(1)
-        except AttributeError:
-            raise PluginError("Unable to find the gomcmd URL in the GOX XML file")
+            dom = xml.dom.minidom.parseString(data)
+        except Exception as err:
+            raise PluginError(("Unable to parse gox file: {0})").format(err))
 
-        match = match.replace("&amp;", "&")
-        match = unquote(match)
+        entries = []
 
-        # SQ and SQTest streams can be gomp2p links, with actual stream address passed as a parameter.
-        if match.startswith("gomp2p://"):
-            match, n = re.subn("^.*LiveAddr=", "", match)
+        for xentry in dom.getElementsByTagName("ENTRY"):
+            entry = {}
+            for child in xentry.childNodes:
+                if isinstance(child, xml.dom.minidom.Element):
+                    if child.tagName == "REF":
+                        href = child.getAttribute("href")
 
-        # Cosmetics, getting rid of the HTML entity, we don't
-        # need either of the " character or &quot;
-        match = match.replace("&quot;", "")
+                        # SQ and SQTest streams can be gomp2p links, with actual stream address passed as a parameter.
+                        if href.startswith("gomp2p://"):
+                            href, n = re.subn("^.*LiveAddr=", "", href)
+                            href = unquote(href)
 
-        return match
+                        entry[child.tagName] = href
+                    else:
+                        entry[child.tagName] = self._get_node_text(child)
+
+            entries.append(entry)
+
+        return entries
 
 __plugin__ = GomTV
